package com.amphinicy.mhp.communication.client;

import java.io.InputStream;
import java.io.*;

public class ControllerMessage {

  /**
   * Default encoding of string
   */
  public static final String ENCODING = "UTF-16";

  /**
   * Type for posting message
   */
  public static final byte POST = 0;

  /**
   * Type for sending message
   */
  public static final byte SEND = 1;

  /**
   * Type for start of session
   */
  public static final byte START = 2;

  /**
   * Type for end of session
   */
  public static final byte END = 3;

  /**
   * Type for response without data
   */
  public static final byte NO_DATA = 4;

  /**
   * Payload type
   */
  byte payloadType;

  /**
   * Payload data
   */
  byte[] payloadData;

  /**
   * Default constructor
   */
  public ControllerMessage() {
  }

  /**
   * Constructor
   * @param payloadData byte[]
   * @param type byte
   */
  public ControllerMessage(byte[] payloadData, byte type) {
    this.payloadData = payloadData;
    this.payloadType = type;
  }

//  public ControllerMessage(byte[] messageData)
//      throws IOException
//  {
//    setMessageData(messageData);
//  }

  /**
   * Constructor
   * @param in InputStream
   * @throws IOException
   */
  public ControllerMessage(InputStream in)
      throws IOException
  {
    read(in);
  }

  public byte[] getPayloadData() {
    return payloadData;
  }

  public void setPayloadType(byte payloadType) {
    this.payloadType = payloadType;
  }

  public void setPayloadData(byte[] payloadData) {
    this.payloadData = payloadData;
  }

  public byte getType() {
    return payloadType;
  }

  public void setMessageData(byte[] messageData)
      throws IOException
  {
    read(new ByteArrayInputStream(messageData));
  }

  public byte[] getMessageData()
  {
    ByteArrayOutputStream byteOut = new ByteArrayOutputStream();
    DataOutputStream dataOut = new DataOutputStream(byteOut);
    try {
      dataOut.writeByte(payloadType);
      dataOut.writeInt(payloadData.length);
      dataOut.write(payloadData);
    }
    catch (IOException ex) {
      return null;
    }
    return byteOut.toByteArray();
  }

  private void read(InputStream in)
      throws IOException
  {
    DataInputStream dataIn = new DataInputStream(in);
    payloadType = dataIn.readByte();
    int size = dataIn.readInt();
    payloadData = new byte[size];
    dataIn.read(payloadData);
  }
}
