package com.amphinicy.mhp.communication.xlet;

import javax.tv.xlet.Xlet;
import com.amphinicy.mhp.communication.client.*;
import org.havi.ui.event.HActionListener;
import javax.tv.xlet.XletContext;
import org.havi.ui.*;
import java.awt.Dimension;
import java.awt.event.KeyEvent;
import java.awt.event.ActionEvent;
import java.io.UnsupportedEncodingException;

public abstract class EchoClient
    implements Xlet, MessageListener, HActionListener {
  private static final String SEND = "Send";

  private XletContext context;
  private HScene scene;

  private Controller controller;

  private HSinglelineEntry inputEntry;
  private HListGroup outputEntry;
  private HTextButton textButton;

  public EchoClient() {
  }

  /**
   * Implementation of Xlet interface (initXlet method).
   * Create scene, components and controller.
   * @param parm1 XletContext
   * @throws XletStateChangeException
   */
  public void initXlet(XletContext parm1) throws javax.tv.xlet.
      XletStateChangeException {
    System.out.println("initXlet(): +");
    this.context = context;

// Put init +++

    HSceneTemplate hst = new HSceneTemplate();
    hst.setPreference(HSceneTemplate.SCENE_SCREEN_DIMENSION,
                      new HScreenDimension(.87f, .87f),
                      HSceneTemplate.PREFERRED);
    hst.setPreference(HSceneTemplate.SCENE_SCREEN_LOCATION,
                      new HScreenPoint(.066f, .066f),
                      HSceneTemplate.PREFERRED);
    scene = HSceneFactory.getInstance().getBestScene(hst);
    scene.setBackgroundMode(HScene.NO_BACKGROUND_FILL);

    inputEntry = new HSinglelineEntry(10, 10, 400, 50, 1024);
    inputEntry.setTextContent("Message text", inputEntry.ALL_STATES);
    scene.add(inputEntry);

    textButton = new HTextButton(SEND, 420, 10, 100, 30);
    textButton.addHActionListener(this);
    textButton.setActionCommand(SEND);
    scene.add(textButton);

    outputEntry = new HListGroup();
    outputEntry.setSelectionMode(false);
    outputEntry.setIconSize(new Dimension(0, 0));
    outputEntry.setLocation(10, 70);
    outputEntry.setSize(400, 400);
    scene.add(outputEntry);

    textButton.setMove(KeyEvent.VK_LEFT, inputEntry);
    inputEntry.setMove(KeyEvent.VK_RIGHT, textButton);
    textButton.setMove(KeyEvent.VK_DOWN, outputEntry);
    inputEntry.setMove(KeyEvent.VK_DOWN, outputEntry);
    outputEntry.setMove(KeyEvent.VK_UP, textButton);

    // Create controller
    controller = createController();
    if (controller == null) {
      outputEntry.addItem(
          new HListElement("Cannot create controller."), 0);
    }

// Put init ---
    scene.setBackgroundMode(HScene.BACKGROUND_FILL);
    scene.setBackground(new org.dvb.ui.DVBColor(31, 24, 107, 250));
    System.out.println("initXlet(): -");
  }

  /**
   * Implementation of Xlet interface (startXlet method).
   * Start xlet, connect to server
   * @throws XletStateChangeException
   */
  public void startXlet() throws javax.tv.xlet.XletStateChangeException {
    System.out.println("startXlet(): +");
    scene.validate();
    scene.setVisible(true);

// Put start +++

    // Connect to server and add message listener
    if (controller != null) {
      try {
        controller.connect();
        controller.addMessageListener(this);
      }
      catch (ControllerException ex) {
        ex.printStackTrace();
        outputEntry.addItem(
            new HListElement("Cannot connect to server."), 0);
      }
    }

    textButton.requestFocus();

// Put start ---

    System.out.println("startXlet(): -");
  }

  /**
   * Implementation of Xlet interface (pauseXlet method).
   */
  public void pauseXlet() {
    System.out.println("pauseXlet(): +");
    scene.setVisible(false);

// Put pause +++

// Put pause ---

    System.out.println("pauseXlet(): -");
  }

  /**
   * Implementation of Xlet interface (destroyXlet method).
   * Destroy xlet, scene and disconnect from server
   * @param parm1 boolean
   * @throws XletStateChangeException
   */
  public void destroyXlet(boolean parm1) throws javax.tv.xlet.
      XletStateChangeException {
    System.out.println("destroyXlet(): + ");

    if (scene != null) {
      HSceneFactory.getInstance().dispose(scene);
      scene = null;
    }

// Put destroy +++
    try {
      // Disconnect from server
      if (controller != null) {
        controller.disconnect();
      }
    }
    catch (ControllerException ex) {
      ex.printStackTrace();
    }

// Put destroy ---

    System.out.println("destroyXlet(): - ");
  }

  /**
   * Implementation of HActionListener
   * @param event ActionEvent
   */
  public void actionPerformed(ActionEvent event) {
    System.out.println("actionPerformed(): +");
    if (event.getActionCommand().equals(SEND)) {
      // SEND button pressed
      String message = inputEntry.getTextContent(inputEntry.NORMAL_STATE);
      System.out.println("Posting message: " + message);
      try {
        ControllerMessage msg = new ControllerMessage();
        msg.setPayloadData(message.getBytes(ControllerMessage.ENCODING));
        // Posting message to server
        controller.post(msg);
      }
      catch (ControllerException ex) {
        ex.printStackTrace();
      }
      catch (Exception ex) {
        ex.printStackTrace();
      }
    }
    System.out.println("actionPerformed(): -");
  }

  /**
   * Implementation of MessageListener
   * @param event MessageEvent
   */
  public void messageArrived(MessageEvent event) {
    System.out.println("messageArrived(): +");
    String message = null;
    // Read payload data from received message
    try {
      message = new String(event.getMessage().getPayloadData(),
                           ControllerMessage.ENCODING);
    }
    catch (UnsupportedEncodingException ex) {
    }
    System.out.println("Received message " + message);
    // Put message into list of messages
    outputEntry.addItem(
        new HListElement(message), 0);
    outputEntry.setCurrentItem(0);
    System.out.println("messageArrived(): -");
  }

  /**
   * Create controller
   * @return Controller
   */
  protected abstract Controller createController();

}
